/**************************************************************************************************************************
// This script was authored by Lorne Brooks - Email: lbtools2k@gmail.com
//
//
// It was inspired by another script called "Frame Range Manager" authored by Borislav "Bobo" Petrov.
//
//
// REVISION HISTORY:
// Version 2.8 24/09/22:
//						- Added a toolbar icon.
//						- Added a link to the online guide in the About section.
//						- Updated FBX export settings to Max 2023.
// Version 2.7 23/08/17:
//						- Ranges created using the script can now have dashes in the name part of the range. Previously this prevented ranges
//							from not loading from text files or scene time tags.
// Version 2.6 08/05/17:
//						- Moved the select ranges dropdown list location to make it easier to find and added a title with a group border.
//						- Moved all the editing options into their own rollout called Edit Ranges (rolled-up by default).
//						- Added a fake border image around the dropdownlist for selecting ranges to help make them more obvious (full UI only).
//						- The main floater and compact dialog monitor positions now save to an INI file, so they remember their previous positions.
//						- The UI now remembers which mode (compact or full) was last used, as its state is now saved to an INI file.
//						- The Edit Ranges rollout will remember its state (rolled-up or rolled-down) by saving it to an INI file. It also adjusts the
//						   height of the floater when the UI is reloaded, as necessary.
//						- Fixed a bug where the compact dialog wasn't closing if the main dialog was reopened.
//						- Fixed a bug where deleting a loaded project caused a crash, by not switching to the next project in the list.
// Version 2.5 25/03/16:
//						- Added a button which opens a new dialog that exposes some optional FBX export settings.
//						- Exporting FBX animation now exports either the whole scene (if nothing is selected) or only the selection for both
//							Ranges and Time Tags options.
// Version 2.4 21/08/15;
//						- When switching between ranges using the dropdown list, if the range name contains a prefix that matches an existing
//							camera in the scene then the current viewport will switch to use that camera. The prefix is denoted by the substring
//							up to the first underscore.
// Version 2.3 27/10/14;
//						- Email change for help contact.
// Version 2.2 10/10/14;
//						- Project and Set dialogs now move with the floater and also affect the position of the floater if they are moved.
// Version 2.1 25/09/14;
//						- Added a new rollout containing functions to split combined animations based on either ranges or time tags
//							into separate maxfiles which are named according to the range name. Keys on the selected objects will be
//							trimmed to the range limits and moved to start at frame 0 with the time range adjusted to fit. Output
//							animation files are saved to the same folder as the original composite scene file.
// Version 2.0 24/09/14;
//						- Changed the FBX file version array to use the correct 2006 string for the parameter setting.
//						- Added an ASCII output option for FBX exporting.
// Version 1.9 23/09/14;
//						- Added a dropdown for choosing which FBX version to use when exporting range or time tag animations.
// Version 1.8 11/09/14;
//						- Added a function to export separate FBX animations based on the loaded ranges.
//						- Added the ability to delete scene time tags.
//						- Added the ability to create a time tag using the interface that's compatible with the other script functions.
// Version 1.7 09/09/14;
//						- Added a function to export separate FBX animations based on time tags in the scene.
//						- Recoded script as a struct to minimize global variables.
// 						- Minor bug fixes.
// Version 1.6 21/09/12;
//						- Now handles user names for local profiles with fullstops in them.
// Version 1.5 07/08/08;
//						UI adjustments...
//						- Widened ranges dropdownlist and allowed more items in the list.
//						- Changed names for reload and delete ranges buttons.
//						- Added style settings to compact version.
// Version 1.4 26/06/08;
//						- Added button F which allows you to add a new range called (FULL_RANGE) with the lowest start time
// 						  and highest end time calculated from the loaded ranges.
// Version 1.3 08/05/08
//						- Added a thin button above the ranges dropdown list which loads a compact dialog showing only the
//						  loaded ranges and reload button. The dialog has the same thin button which reloads the TRM floater.
// Version 1.2 07/05/08;
//						- When loading ranges from scene time tags or a text file, script now handles invalid formats.
//						- Help button ? added to Time Tags rollout to show correct formatting.
// Version 1.1 02/05/08;
//						- Fixed bug with sorting sets when scene time tags copied to ranges.
// Version 1.0 01/05/08;
//						- First public release
//
//
// Notes:
//		- Tooltip comments appear for all buttons as a form of help when moused over.
//		- The script makes use of INI files to store data in the user plugins config folder.
//		- Time ranges are saved as a collection called a Set. Multiple sets are saved as a collection called Projects.
//		- The Time Tags rollout allows transfer of data from a text file to scene time tags to the loaded ranges.
//		- Delete buttons are inactive when only one Project/Set/Range is left, so that there's at least one available.
//
//
// This script is not copyrighted, please feel free to modify or distribute.
//
**************************************************************************************************************************/

global g_LBTools_TRM

struct s_LBTools_timeRangeManager
(

/*==================================================================================
// Structure variables
====================================================================================*/

timeRangeFloater,
RangeCollections_Rollout,
EditRanges_Rollout,
SetsManager_Rollout,
ProjectsManager_Rollout,
timeTags_Rollout,
CompactRanges_Rollout,
FBXsettings_Rollout,
FBXexportAnim_Rollout,
SeparateMaxfiles_Rollout,
about_Rollout,
savedFile,
loadedFile,
TRM_pos, --// Floater position
SM_pos, --// Sets manager dialog position
PM_pos, --// Projects manager dialog position
f_initializeTRM_IniFile,
f_initializeProjects_IniFile,
f_openTimeRangeManager,
f_openSetsManager,
f_openProjectsManager,
f_openCompactDialog,
f_getRangesFromFile,
f_getSetsFromFile,
f_sortSets,
f_validateFileTags,
f_validateSceneTags,
f_findFullRange,
f_switchCamera,
f_saveSceneTimeTags,
f_loadSceneTimeTags,
a_sets = #(),
a_setTags = #(),
a_setItemNames = #(),
a_setItemRanges = #(),
setTag = "Set_1",
selectedTag = "Set_1",
setName = "(DEFAULT_SET)",
projectName = "(DEFAULT_PROJECT)",
currentRange = "1",
rangeName = "(DEFAULT_RANGE)",
a_ranges = #(),
projectIniFile = getDir #plugcfg + @"\TRM#(DEFAULT_PROJECT).ini",
trmINIFile = getDir #plugcfg + @"\TimeRangeManager.ini",
FBXexportINI = getDir #plugcfg + @"\TRM_FBXexport.ini",
imageFile1 = getDir #usericons + @"\LBTools_TRM_Image1.bmp",

/*==================================================================================
// Structure functions
/*==================================================================================

f_createFBXexportINI
f_initializeFBXexport
f_timeRangeManager

/*==================================================================================*/

fn f_createFBXexportINI =
(
	--// Setup the default settings on first time creation of the INI file
	if NOT (doesFileExist g_LBTools_TRM.FBXexportINI) do
	(
		--// Version
		setIniSetting g_LBTools_TRM.FBXexportINI "FBX Export Settings" "Version" "FBX202000"
		setIniSetting g_LBTools_TRM.FBXexportINI "FBX Export Settings" "ASCII" "false"
		--// Geometry
		setIniSetting g_LBTools_TRM.FBXexportINI "FBX Export Settings" "SmoothingGroups" "true"
		setIniSetting g_LBTools_TRM.FBXexportINI "FBX Export Settings" "NormalsPerPoly" "false" --// Split Per-Vertex Normals --// NOTE: If this is TRUE, Unreal will display a warning about smoothing groups when importing
		setIniSetting g_LBTools_TRM.FBXexportINI "FBX Export Settings" "TangentSpaceExport" "false" --// Tangents and Binormals
		setIniSetting g_LBTools_TRM.FBXexportINI "FBX Export Settings" "GeomAsBone" "true"
		setIniSetting g_LBTools_TRM.FBXexportINI "FBX Export Settings" "Triangulate" "true"
		setIniSetting g_LBTools_TRM.FBXexportINI "FBX Export Settings" "PreserveEdgeOrientation" "true"
		--// Deformations
		setIniSetting g_LBTools_TRM.FBXexportINI "FBX Export Settings" "Skin" "true"
		setIniSetting g_LBTools_TRM.FBXexportINI "FBX Export Settings" "Shape" "true" --// Export the Morpher modifier "blendshapes" -- same as the Morph option in FBX options > Deformations
		--// Misc
		setIniSetting g_LBTools_TRM.FBXexportINI "FBX Export Settings" "Cameras" "false"
		setIniSetting g_LBTools_TRM.FBXexportINI "FBX Export Settings" "Lights" "false"
		setIniSetting g_LBTools_TRM.FBXexportINI "FBX Export Settings" "EmbedTextures" "false"
		setIniSetting g_LBTools_TRM.FBXexportINI "FBX Export Settings" "CAT2HIK" "false"
	)
), --// end function

/*----------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------*/

fn f_initializeFBXexport =
(
	fbxExporterSetParam "ExportAnimationOnly" false --// This must be false for Unreal importing!
	fbxExporterSetParam "Animation" true
	fbxExporterSetParam "ASCII" (execute(getIniSetting g_LBTools_TRM.FBXexportINI "FBX Export Settings" "ASCII"))
	fbxExporterSetParam "AxisConversionMethod" "Animation"
	fbxExporterSetParam "BakeAnimation" true
	fbxExporterSetParam "BakeframeStart" animationrange.start
	fbxExporterSetParam "BakeframeEnd" animationrange.end
	fbxExporterSetParam "BakeframeStep" 1
	fbxExporterSetParam "BakeResampleAnimation" true
	fbxExporterSetParam "Cameras" (execute(getIniSetting g_LBTools_TRM.FBXexportINI "FBX Export Settings" "Cameras"))
	fbxExporterSetParam "CAT2HIK" (execute(getIniSetting g_LBTools_TRM.FBXexportINI "FBX Export Settings" "CAT2HIK"))
	fbxExporterSetParam "ColladaTriangulate" false
	fbxExporterSetParam "ColladaSingleMatrix" false
	fbxExporterSetParam "ColladaFrameRate" 30.0
	fbxExporterSetParam "Convert2Tiff" false
	fbxExporterSetParam "ConvertUnit" "cm"
	fbxExporterSetParam "EmbedTextures" (execute(getIniSetting g_LBTools_TRM.FBXexportINI "FBX Export Settings" "EmbedTextures"))
	fbxExporterSetParam "FileVersion" (getIniSetting g_LBTools_TRM.FBXexportINI "FBX Export Settings" "Version")
	fbxExporterSetParam "FilterKeyReducer" false
	fbxExporterSetParam "GeomAsBone" (execute(getIniSetting g_LBTools_TRM.FBXexportINI "FBX Export Settings" "GeomAsBone"))
	fbxExporterSetParam "GenerateLog" false
	fbxExporterSetParam "Lights" (execute(getIniSetting g_LBTools_TRM.FBXexportINI "FBX Export Settings" "Lights"))
	--fbxExporterSetParam "LoadExportPresetFile"
	fbxExporterSetParam "NormalsPerPoly" (execute(getIniSetting g_LBTools_TRM.FBXexportINI "FBX Export Settings" "NormalsPerPoly"))
	fbxExporterSetParam "PointCache" false
	--fbxExporterSetParam "PopSettings"
	fbxExporterSetParam "PreserveInstances" false
	--fbxExporterSetParam "PushSettings"
	fbxExporterSetParam "RemoveSingleKeys" false
	fbxExporterSetParam "Resampling" 30.0
	--fbxExporterSetParam "ResetExport"
	fbxExporterSetParam "ScaleFactor" 1.0
	--fbxExporterSetParam "SelectionSet"
	fbxExporterSetParam "SelectionSetExport" false
	fbxExporterSetParam "Shape" (execute(getIniSetting g_LBTools_TRM.FBXexportINI "FBX Export Settings" "Shape")) --// Export the Morpher modifier "blendshapes" -- same as the Morph option in FBX options > Deformations
	fbxExporterSetParam "Skin" (execute(getIniSetting g_LBTools_TRM.FBXexportINI "FBX Export Settings" "Skin"))
	fbxExporterSetParam "ShowWarnings" true
	fbxExporterSetParam "SmoothingGroups" (execute(getIniSetting g_LBTools_TRM.FBXexportINI "FBX Export Settings" "SmoothingGroups"))
	fbxExporterSetParam "SmoothMeshExport" false
	--fbxExporterSetParam "SplitAnimationIntoTakes"
	fbxExporterSetParam "TangentSpaceExport" (execute(getIniSetting g_LBTools_TRM.FBXexportINI "FBX Export Settings" "TangentSpaceExport"))
	fbxExporterSetParam "Triangulate" (execute(getIniSetting g_LBTools_TRM.FBXexportINI "FBX Export Settings" "Triangulate"))
	fbxExporterSetParam "UpAxis" "Y"
	fbxExporterSetParam "UseSceneName" false
	fbxExporterSetParam "PreserveEdgeOrientation" (execute(getIniSetting g_LBTools_TRM.FBXexportINI "FBX Export Settings" "PreserveEdgeOrientation"))
), --// end function

/*----------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------*/

fn f_timeRangeManager =
(
	fn f_validateFileTags = --// Checks whether the format for ranges in a text file are valid for loading
	(
		local ttFile = openFile g_LBTools_TRM.loadedFile
		while NOT eof ttFile do
		(
			local a_lineStr = #()
			local a_rangeStr = #()
			lineStr = readLine ttFile
			a_lineStr = filterString lineStr " "
			if a_lineStr.count != 2 do return false
			a_rangeStr = filterString a_lineStr[2] "-"
			if execute a_rangeStr[1] == undefined do return false
			if execute a_rangeStr[2] == undefined do return false
			if (a_rangeStr[1] as integer) >= (a_rangeStr[2] as integer) do return false
		)
		close ttFile
		return true
	)

	fn f_validateSceneTags nTags = --// Checks whether the format for time tags in a scene are valid for loading
	(
		for i = 1 to nTags do
		(
			local id = FrameTagManager.GetTagID i
			local tagName = FrameTagManager.GetNameByID id
			local a_tagStr = #()
			local a_rangeStr = #()
			a_tagStr = filterString tagName " "
			if a_tagStr.count != 2 do return false
			a_rangeStr = filterString a_tagStr[2] "-"
			if execute a_rangeStr[1] == undefined do return false
			if execute a_rangeStr[2] == undefined do return false
			if (a_rangeStr[1] as integer) >= (a_rangeStr[2] as integer) do return false
		)
		return true
	)

	fn f_saveSceneTimeTags nTags =
	(
		try
		(
			local ttFile = openFile g_LBTools_TRM.savedFile mode:"w"
			for i = 1 to nTags do
			(
				--// Get each tag name in turn
				id = FrameTagManager.GetTagID i
				tagName = FrameTagManager.GetNameByID id
				--// Write to text file
				format "%\n" tagName to:ttFile
			)
			close ttFile
			ShellLaunch g_LBTools_TRM.savedFile ""
		) catch (messageBox "Error: Could not open file for writing." title:"")
	)
	
	fn f_loadSceneTimeTags =
	(
		if (queryBox "This process will load all time tags from the text file, replacing any existing ones... continue?" title:"") do
		(
			FrameTagManager.ResetFrameTags()
			local ttFile = openFile g_LBTools_TRM.loadedFile
			while NOT eof ttFile do
			(
				local a_lineStr = #()
				local a_rangeStr = #()
				lineStr = readLine ttFile
				a_lineStr = filterString lineStr " "
				a_rangeStr = filterString a_lineStr[2] "-"
				local nameStr = a_lineStr[1] + " " + a_rangeStr[1] + "-" + a_rangeStr[2]
				FrameTagManager.CreateNewTag nameStr (a_rangeStr[1] as time)
			)
			close ttFile
			messageBox "Process complete... all time tags in scene replaced." title:""
		)
	)

	fn f_getRangesFromFile setTagIn = --// Returns a list of all ranges for the set-tag passed as a parameter
	(
		local numRanges = execute (getIniSetting g_LBTools_TRM.projectIniFile setTagIn "RangeCount")
		if numRanges == OK do numRanges = 0
		local a_rangeStrings = #()
		g_LBTools_TRM.a_ranges = #()
		for i = 1 to numRanges do
		(
    		local rangeRead = getIniSetting g_LBTools_TRM.projectIniFile setTagIn ("Range_" + i as string)
			a_range = filterstring rangeRead ":"
    		g_LBTools_TRM.rangeName = a_range[1]; startFrame = a_range[2]; endFrame = a_range[3]
			if startFrame == OK then startFrame = 0
			if endFrame == OK then endFrame = 100
			if g_LBTools_TRM.rangeName == "" then g_LBTools_TRM.rangeName = "No Name"
			append g_LBTools_TRM.a_ranges #(g_LBTools_TRM.rangeName, startFrame, endFrame)
    		append a_rangeStrings (g_LBTools_TRM.rangeName + "  [" + startFrame as string + " - " + endFrame as string + "]")
		)
		return a_rangeStrings
	)

	fn f_getSetsFromFile theIniFile = --// Returns an array of sets and their associated array of ranges
	(
		local numSets = execute (getIniSetting theIniFile "Sets" "SetCount")
		if numSets == OK do numSets = 0
		local a_setsReturn = #()
		g_LBTools_TRM.a_setTags = getIniSetting theIniFile
		deleteItem g_LBTools_TRM.a_setTags 1
		for i = 1 to numSets do
		(
			local a_setRead = getIniSetting theIniFile ("Set_" + i as string)
			local nameStr = getIniSetting theIniFile ("Set_" + i as string) a_setRead[1]
			local a_rangesFound = #()
			--// Get all the ranges for this set
			a_rangesFound = g_LBTools_TRM.f_getRangesFromFile g_LBTools_TRM.a_setTags[i]
			append a_setsReturn #(nameStr, a_rangesFound)
		)
		return a_setsReturn
	)

	fn f_sortSets =
	(
		--// Back up the contents of remaining sets
		local a_setTagsBackup = #()
		a_setTagsBackup = getIniSetting g_LBTools_TRM.projectIniFile
		--// Create a backup of all the remaining sets and their range contents
		local a_setContents = #()
		for i = 2 to a_setTagsBackup.count do
		(
			--// Collect all the ranges for each set in turn
			local a_rangeTags = #()
			a_rangeTags = getIniSetting g_LBTools_TRM.projectIniFile a_setTagsBackup[i]
			local a_rangeContents = #()
			for j = 1 to a_rangeTags.count do append a_rangeContents (getIniSetting g_LBTools_TRM.projectIniFile a_setTagsBackup[i] a_rangeTags[j])
			--// Add ranges to the sets backup
			append a_setContents a_rangeContents
		)
		--// Remove all sets from INI file
		for i = 2 to a_setTagsBackup.count do delIniSetting g_LBTools_TRM.projectIniFile a_setTagsBackup[i]
		--// Rebuild the sets again using the contents backup contents array
		for j = 1 to a_setContents.count do
		(
			setIniSetting g_LBTools_TRM.projectIniFile ("Set_" + j as string) "SetName" (a_setContents[j][1] as string)
			setIniSetting g_LBTools_TRM.projectIniFile ("Set_" + j as string) "CurrentRange" (a_setContents[j][2] as string)
			setIniSetting g_LBTools_TRM.projectIniFile ("Set_" + j as string) "RangeCount" (a_setContents[j][3] as string)
			--// For each remaining item in the array add a new range entry
			local rangeNum = 1
			for k = 4 to a_setContents[j].count do
			(
				setIniSetting g_LBTools_TRM.projectIniFile ("Set_" + j as string) ("Range_" + rangeNum as string) a_setContents[j][k]
				rangeNum += 1
			)
		)
		return a_setContents
	)

	fn f_findFullRange =
	(
		local a_minTimes = #()
		local a_maxTimes = #()
		local a_rangeTimes = #()
		a_minTimes = for range in g_LBTools_TRM.a_ranges collect (range[2] as number)
		append a_rangeTimes ((amin a_minTimes) as string)
		a_maxTimes = for range in g_LBTools_TRM.a_ranges collect (range[3] as number)
		append a_rangeTimes ((amax a_maxTimes) as string)
		return a_rangeTimes
	)

	fn f_switchCamera str_range =
	(
		local a_rangeStr = filterString str_range "_"
		for c in cameras where superClassOf c == camera do
		(
			if c.name == a_rangeStr[1] do
			(
				if viewport.CanSetToViewport c do
				(
					viewport.setCamera c
					exit
				)
			)
		)
	)
	
	-----------------------------------------------------------------------------------------
	--// Rollout definitions
	-----------------------------------------------------------------------------------------

	rollout SeparateMaxfiles_Rollout "Separate Maxfiles"
	(
		label lbl_1 "Based on a directly keyed selection."
		button btn_separateMaxfilesRanges "Separate > Ranges" width:200 tooltip:"Uses the currently loaded ranges data to split animations into separate maxfiles based on the range name. Requires a selection of directly animated objects to work."
		button btn_separateMaxfilesTimeTags "Separate > Time Tags" width:200 tooltip:"Uses the correctly formatted time tags data to split animations into separate maxfiles based on the tag name. Requires a selection of directly animated objects to work."
		checkbox chkbx_delTags "Delete all time tags in output files" checked:true tooltip:"All time tags will be removed on the saved animation maxfiles (not the original source file)."
		
		fn f_removeKeys a_objs =
		(
			for obj in a_objs do
			(
				selectKeys obj.controller animationRange
				if classof obj == Biped_Object then
				(
					if classof obj.controller == Vertical_Horizontal_Turn then --// Biped COM
					(
						biped.deleteKeys (biped.getHorizontalControl obj.controller) #selection
						biped.deleteKeys (biped.getVerticalControl obj.controller) #selection
						biped.deleteKeys (biped.getTurnControl obj.controller) #selection
					)
					else --// Other Biped parts
					(
						biped.deleteKeys obj.controller #selection
					)
				)
				else --// Non Biped
				(
					deleteKeys obj.controller #selection
				)
			)
		)
		
		on btn_separateMaxfilesRanges pressed do
		(
			try --// Try/Catch in case ranges are invalid
			(
				local a_objs = selection as array
				local a_objNames = #()
				for obj in a_objs do
				(
					append a_objNames obj.name
				)
				if a_objs.count > 0 then
				(
					local str_origMaxfile = (maxFilePath + maxFileName)
					for a_rangeItem in g_LBTools_TRM.a_ranges do
					(
						--// Due to fetching the objects are considered deleted, so find them again
						a_objs = #()
						for str in a_objNames do
						(
							local objFound = (getNodeByName str)
							if classof objFound != CATParent do
							(
								append a_objs objFound
							)
						)
						
						--// Gather the range data
						local str_rangeName = a_rangeItem[1]
						animationRange = interval (a_rangeItem[2] as time) (a_rangeItem[3] as time)

						--// Remove keys on selected objects outside the range
						if (a_rangeItem[2] as integer) > 0 do
						(
							animationRange = interval 0 ((a_rangeItem[2] as time)-1)
							f_removeKeys a_objs
						)
						animationRange = interval ((a_rangeItem[3] as time)+1) (10000 as time) --// Hopefully there's no keys beyond 10000f ?!
						f_removeKeys a_objs
						
						--// Select all the keys on the selected objects
						animationRange = interval 0 (a_rangeItem[3] as time)
						for obj in a_objs do
						(
							selectKeys obj.controller animationRange
						)
						
						--// Collect the Biped parts (if any), for moving keys.
						--// As Biped tracks (by default) are usually not "Separate FK Tracks" for shared parts (like limbs, head and spine), you only want to move one part for each
						local bRoot
						for obj in a_objs where (classof obj.controller == Vertical_Horizontal_Turn) do
						(
							bRoot = obj
							exit
						)
						local a_bipedParts = #()
						if bRoot != undefined do
						(
							bPelvis = biped.getNode bRoot #Pelvis
							bSpine = biped.getNode bRoot #Spine link:2
							bHead = biped.getNode bRoot #Head
							bHandL = biped.getNode bRoot #lArm link:4
							bHandR = biped.getNode bRoot #rArm link:4
							bFootL = biped.getNode bRoot #lLeg link:3
							bFootR = biped.getNode bRoot #rLeg link:3
							a_bipedParts = #(bRoot, bPelvis, bSpine, bHead, bHandL, bHandR, bFootL, bFootR)
						)

						--// Move all keys to start at frame 0 for the length of the initial range (offset amount)
						local offsetAmount = -(a_rangeItem[2] as integer)
						if bRoot != undefined then
						(
							for bp in a_bipedParts do
							(
								moveKeys bp.controller offsetAmount #selection
							)
						)
						else
						(
							for obj in a_objs do
							(
								moveKeys obj.controller offsetAmount #selection
							)
						)
						
						--// Set the end range to the length of the time tag
						animationRange = interval 0 ((a_rangeItem[3] as time) - (a_rangeItem[2] as time))
						
						--// Deselect all keys and save the maxfile
						for obj in a_objs do
						(
							deselectKeys obj.controller animationRange
						)
						local str_exportFileName = (maxFilePath + a_rangeItem[1] + ".max")
						saveMaxFile str_exportFileName
						loadMaxFile str_origMaxfile quiet:true
					)
					messageBox "Maxfiles have been separated to the scene folder based on the stored ranges." title:""
				)
				else
				(
					messageBox "Error: Select some animated objects to adjust keys for." title:""
				)
			)catch(messagebox "Error: Ranges may be invalid." title:"")
		)
		
		on btn_separateMaxfilesTimeTags pressed do
		(
			try --// Try/Catch in case time tags are invalid
			(
				local a_objs = selection as array
				local a_objNames = #()
				for obj in a_objs do
				(
					append a_objNames obj.name
				)
				if a_objs.count > 0 then
				(
					local ttCount = FrameTagManager.GetTagCount()
					if ttCount > 0 then
					(
						local str_origMaxfile = (maxFilePath + maxFileName)
						for i = 1 to ttCount do
						(
							--// Due to fetching the objects are considered deleted, so find them again
							a_objs = #()
							for str in a_objNames do
							(
								local objFound = (getNodeByName str)
								if classof objFound != CATParent do
								(
									append a_objs objFound
								)
							)
							
							--// Gather the time tag data
							local str_tt = FrameTagManager.GetNameByID i
							local a_tt = (filterString str_tt " ")
							local str_tagName = a_tt[1]
							local a_range = (filterString a_tt[2] "-")

							--// Remove keys on selected objects outside the range
							if (a_range[1] as integer) > 0 do
							(
								animationRange = interval 0 ((a_range[1] as time)-1)
								f_removeKeys a_objs
							)
							animationRange = interval ((a_range[2] as time)+1) (10000 as time) --// Hopefully there's no keys beyond 10000f ?!
							f_removeKeys a_objs
							
							--// Select all the keys on the selected objects
							animationRange = interval 0 (a_range[2] as time)
							for obj in a_objs do
							(
								selectKeys obj.controller animationRange
							)
							
							--// Collect the Biped parts (if any), for moving keys.
							--// As Biped tracks (by default) are usually not "Separate FK Tracks" for shared parts (like limbs, head and spine), you only want to move one part for each
							local bRoot
							for obj in a_objs where (classof obj.controller == Vertical_Horizontal_Turn) do
							(
								bRoot = obj
								exit
							)
							local a_bipedParts = #()
							if bRoot != undefined do
							(
								bPelvis = biped.getNode bRoot #Pelvis
								bSpine = biped.getNode bRoot #Spine link:2
								bHead = biped.getNode bRoot #Head
								bHandL = biped.getNode bRoot #lArm link:4
								bHandR = biped.getNode bRoot #rArm link:4
								bFootL = biped.getNode bRoot #lLeg link:3
								bFootR = biped.getNode bRoot #rLeg link:3
								a_bipedParts = #(bRoot, bPelvis, bSpine, bHead, bHandL, bHandR, bFootL, bFootR)
							)

							--// Move all keys to start at frame 0 for the length of the initial time tag (offset amount)
							local offsetAmount = -(a_range[1] as integer)
							if bRoot != undefined then
							(
								for bp in a_bipedParts do
								(
									moveKeys bp.controller offsetAmount #selection
								)
							)
							else
							(
								for obj in a_objs do
								(
									moveKeys obj.controller offsetAmount #selection
								)
							)
							
							--// Set the end range to the length of the time tag
							animationRange = interval 0 ((a_range[2] as time) - (a_range[1] as time))
							
							--// Deselect all keys, remove tags (optional) and save the maxfile
							for obj in a_objs do
							(
								deselectKeys obj.controller animationRange
							)
							if chkbx_delTags.checked do
							(
								FrameTagManager.ResetFrametags()
							)
							local str_exportFileName = (maxFilePath + str_tagName + ".max")
							saveMaxFile str_exportFileName
							loadMaxFile str_origMaxfile quiet:true
						)
						messageBox "Maxfiles have been separated to the scene folder based on the time tags." title:""
					)
					else
					(
						messageBox "Error: No time tags found in the scene." title:""
					)
				)
				else
				(
					messageBox "Error: Select some animated objects to adjust keys for." title:""
				)
			)catch(messagebox "Error: Time tags may be invalid." title:"")
		)
	) --// end rollout
	
	rollout FBXsettings_Rollout "TRM - FBX Export Settings"
	(
		local a_fbxVersion = #("#FBX202000","#FBX201900","FBX201800")
		
		label lbl_1 "File Version:" align:#left
		dropdownlist ddl_fbxVersion "" items:#("FBX 2020","FBX 2019","FBX 2018") width:120 across:2 offset:[20,0] tooltip:"Choose which FBX version to output as"
		checkbox chkbx_fbxASCII "ASCII output (OFF for binary)" checked:false align:#left offset:[-15,3] tooltip:"Output the FBX file in ASCII format for easy reading, otherwise output as binary"
		label lbl_2 "Geometry:" align:#left
		checkbox chkbx_smoothGroups "Smoothing Groups" offset:[20,0]
		checkbox chkbx_splitPerVertex "Split Per-Vertex Normals" offset:[20,0]
		checkbox chkbx_tangentBinormals "Tangent and Binormals" offset:[20,0]
		checkbox chkbx_geomAsBone "Convert Deforming Dummies to Bones" offset:[20,0]
		checkbox chkbx_triangulate "Triangulate" offset:[20,0]
		checkbox chkbx_preserveEdges "Preserver Edge Orientation" offset:[20,0]	
		label lbl_3 "Deformations:" align:#left
		checkbox chkbx_skin "Skins" offset:[20,0]
		checkbox chkbx_shape "Morphs" offset:[20,0]
		label lbl_4 "Misc:" align:#left
		checkbox chkbx_cameras "Cameras" offset:[20,0]
		checkbox chkbx_lights "Lights" offset:[20,0]
		checkbox chkbx_embedTextures "Embed Media" offset:[20,0]
		checkbox chkbx_CAT2HIK "CAT Characters as HIK" offset:[20,0]
		label lbl_gap ""
		label lbl_5 "NOTE: Animation settings such as the output name and range" align:#left
		label lbl_6 "are controlled by Time Range Manager based on the ranges" align:#left
		label lbl_7 "or time tags you create." align:#left
		
		on FBXsettings_Rollout open do
		(
			if execute(getIniSetting g_LBTools_TRM.FBXexportINI "FBX Export Settings" "SmoothingGroups") then
			(chkbx_smoothGroups.checked = true)else(chkbx_smoothGroups.checked = false)
			
			if execute(getIniSetting g_LBTools_TRM.FBXexportINI "FBX Export Settings" "NormalsPerPoly") then
			(chkbx_splitPerVertex.checked = true)else(chkbx_splitPerVertex.checked = false)
			
			if execute(getIniSetting g_LBTools_TRM.FBXexportINI "FBX Export Settings" "TangentSpaceExport") then
			(chkbx_tangentBinormals.checked = true)else(chkbx_tangentBinormals.checked = false)
			
			if execute(getIniSetting g_LBTools_TRM.FBXexportINI "FBX Export Settings" "GeomAsBone") then
			(chkbx_geomAsBone.checked = true)else(chkbx_geomAsBone.checked = false)
			
			if execute(getIniSetting g_LBTools_TRM.FBXexportINI "FBX Export Settings" "Triangulate") then
			(chkbx_triangulate.checked = true)else(chkbx_triangulate.checked = false)
			
			if execute(getIniSetting g_LBTools_TRM.FBXexportINI "FBX Export Settings" "PreserveEdgeOrientation") then
			(chkbx_preserveEdges.checked = true)else(chkbx_preserveEdges.checked = false)
			
			if execute(getIniSetting g_LBTools_TRM.FBXexportINI "FBX Export Settings" "Skin") then
			(chkbx_skin.checked = true)else(chkbx_skin.checked = false)
			
			if execute(getIniSetting g_LBTools_TRM.FBXexportINI "FBX Export Settings" "Shape") then
			(chkbx_shape.checked = true)else(chkbx_shape.checked = false)
			
			if execute(getIniSetting g_LBTools_TRM.FBXexportINI "FBX Export Settings" "Cameras") then
			(chkbx_cameras.checked = true)else(chkbx_cameras.checked = false)			
			
			if execute(getIniSetting g_LBTools_TRM.FBXexportINI "FBX Export Settings" "Lights") then
			(chkbx_lights.checked = true)else(chkbx_lights.checked = false)
			
			if execute(getIniSetting g_LBTools_TRM.FBXexportINI "FBX Export Settings" "EmbedTextures") then
			(chkbx_embedTextures.checked = true)else(chkbx_embedTextures.checked = false)
			
			if execute(getIniSetting g_LBTools_TRM.FBXexportINI "FBX Export Settings" "CAT2HIK") then
			(chkbx_CAT2HIK.checked = true)else(chkbx_CAT2HIK.checked = false)
			
			local str_fbxVersion = getIniSetting g_LBTools_TRM.FBXexportINI "FBX Export Settings" "Version"
			for index = 1 to a_fbxVersion.count do
			(
				if (findString a_fbxVersion[index] str_fbxVersion) == 1 do
				(
					ddl_fbxVersion.selection = index
					exit
				)
			)
			
			if execute(getIniSetting g_LBTools_TRM.FBXexportINI "FBX Export Settings" "ASCII") then
			(chkbx_fbxASCII.checked = true)else(chkbx_fbxASCII.checked = false)
		)
		
		on chkbx_smoothGroups changed theState do
		(
			if theState then
			(
				setIniSetting g_LBTools_TRM.FBXexportINI "FBX Export Settings" "SmoothingGroups" "true"
			)
			else
			(
				setIniSetting g_LBTools_TRM.FBXexportINI "FBX Export Settings" "SmoothingGroups" "false"
			)
		)
		
		on chkbx_splitPerVertex changed theState do
		(
			if theState then
			(
				setIniSetting g_LBTools_TRM.FBXexportINI "FBX Export Settings" "NormalsPerPoly" "true"
			)
			else
			(
				setIniSetting g_LBTools_TRM.FBXexportINI "FBX Export Settings" "NormalsPerPoly" "false"
			)
		)
		
		on chkbx_tangentBinormals changed theState do
		(
			if theState then
			(
				setIniSetting g_LBTools_TRM.FBXexportINI "FBX Export Settings" "TangentSpaceExport" "true"
			)
			else
			(
				setIniSetting g_LBTools_TRM.FBXexportINI "FBX Export Settings" "TangentSpaceExport" "false"
			)
		)
		
		on chkbx_geomAsBone changed theState do
		(
			if theState then
			(
				setIniSetting g_LBTools_TRM.FBXexportINI "FBX Export Settings" "GeomAsBone" "true"
			)
			else
			(
				setIniSetting g_LBTools_TRM.FBXexportINI "FBX Export Settings" "GeomAsBone" "false"
			)
		)
		
		on chkbx_triangulate changed theState do
		(
			if theState then
			(
				setIniSetting g_LBTools_TRM.FBXexportINI "FBX Export Settings" "Triangulate" "true"
			)
			else
			(
				setIniSetting g_LBTools_TRM.FBXexportINI "FBX Export Settings" "Triangulate" "false"
			)
		)
		
		on chkbx_preserveEdges changed theState do
		(
			if theState then
			(
				setIniSetting g_LBTools_TRM.FBXexportINI "FBX Export Settings" "PreserveEdgeOrientation" "true"
			)
			else
			(
				setIniSetting g_LBTools_TRM.FBXexportINI "FBX Export Settings" "PreserveEdgeOrientation" "false"
			)
		)
		
		on chkbx_skin changed theState do
		(
			if theState then
			(
				setIniSetting g_LBTools_TRM.FBXexportINI "FBX Export Settings" "Skin" "true"
			)
			else
			(
				setIniSetting g_LBTools_TRM.FBXexportINI "FBX Export Settings" "Skin" "false"
			)
		)
		
		on chkbx_shape changed theState do
		(
			if theState then
			(
				setIniSetting g_LBTools_TRM.FBXexportINI "FBX Export Settings" "Shape" "true"
			)
			else
			(
				setIniSetting g_LBTools_TRM.FBXexportINI "FBX Export Settings" "Shape" "false"
			)
		)
		
		on chkbx_cameras changed theState do
		(
			if theState then
			(
				setIniSetting g_LBTools_TRM.FBXexportINI "FBX Export Settings" "Cameras" "true"
			)
			else
			(
				setIniSetting g_LBTools_TRM.FBXexportINI "FBX Export Settings" "Cameras" "false"
			)
		)
		
		on chkbx_lights changed theState do
		(
			if theState then
			(
				setIniSetting g_LBTools_TRM.FBXexportINI "FBX Export Settings" "Lights" "true"
			)
			else
			(
				setIniSetting g_LBTools_TRM.FBXexportINI "FBX Export Settings" "Lights" "false"
			)
		)
		
		on chkbx_embedTextures changed theState do
		(
			if theState then
			(
				setIniSetting g_LBTools_TRM.FBXexportINI "FBX Export Settings" "EmbedTextures" "true"
			)
			else
			(
				setIniSetting g_LBTools_TRM.FBXexportINI "FBX Export Settings" "EmbedTextures" "false"
			)
		)
		
		on chkbx_CAT2HIK changed theState do
		(
			if theState then
			(
				setIniSetting g_LBTools_TRM.FBXexportINI "FBX Export Settings" "CAT2HIK" "true"
			)
			else
			(
				setIniSetting g_LBTools_TRM.FBXexportINI "FBX Export Settings" "CAT2HIK" "false"
			)
		)
		
		on ddl_fbxVersion selected val do
		(
			setIniSetting g_LBTools_TRM.FBXexportINI "FBX Export Settings" "Version" a_fbxVersion[val]
		)
		
		on chkbx_fbxASCII changed theState do
		(
			if theState then
			(
				setIniSetting g_LBTools_TRM.FBXexportINI "FBX Export Settings" "ASCII" "true"
			)
			else
			(
				setIniSetting g_LBTools_TRM.FBXexportINI "FBX Export Settings" "ASCII" "false"
			)
		)
	) --// end rollout
	
	rollout FBXexportAnim_Rollout "FBX Export Animation"
	(
		button btn_openFBXexportSettings "###  FBX Export Settings  ###" width:200 tooltip:"Display a separate dialog for changing some of the FBX export settings"
		button btn_exportRanges "Export > Ranges" width:200 tooltip:"Export separate animation files to FBX format based on the currently loaded ranges which specify the name, start and end time for each sequence"
		button btn_exportTimeTags "Export > Time Tags" width:200 tooltip:"Export separate animation files to FBX format based on the scene time tags which specify the name, start and end time for each sequence"
		
		on btn_openFBXexportSettings pressed do
		(
			try(destroyDialog g_LBTools_TRM.FBXsettings_Rollout)catch()
			createDialog g_LBTools_TRM.FBXsettings_Rollout 350 415
		)
		
		fn f_exportRanges isSelected: =
		(
			holdMaxFile()
			for a_rangeItem in g_LBTools_TRM.a_ranges do
			(
				local str_rangeName = a_rangeItem[1]
				animationRange = interval (a_rangeItem[2] as time) (a_rangeItem[3] as time)
				local str_exportFileName = (maxFilePath + str_rangeName + ".FBX")
				g_LBTools_TRM.f_initializeFBXexport()
				exportFile str_exportFileName #noPrompt selectedOnly:isSelected using:FBXEXP
			)
			fetchMaxFile quiet:true
			messageBox "Animations have been exported to the scene folder based on the loaded ranges." title:""
		)
		
		on btn_exportRanges pressed do
		(
			try
			(
				local a_objs = selection as array
				if a_objs.count > 0 then --// Export only what is selected
				(
					f_exportRanges isSelected:true
				)
				else --// Export the whole scene
				(
					f_exportRanges isSelected:false
				)
			)catch(messagebox "Error: Ranges may be invalid." title:"")
		)
		
		fn f_exportTimeTags isSelected: =
		(
			local ttCount = FrameTagManager.GetTagCount()
			if ttCount > 0 then
			(
				holdMaxFile()
				for i = 1 to ttCount do
				(
					local str_tt = FrameTagManager.GetNameByID i
					local a_tt = (filterString str_tt " ")
					local str_tagName = a_tt[1]
					local a_range = (filterString a_tt[2] "-")
					animationRange = interval (a_range[1] as time) (a_range[2] as time)
					local str_exportFileName = (maxFilePath + str_tagName + ".FBX")
					g_LBTools_TRM.f_initializeFBXexport()
					exportFile str_exportFileName #noPrompt selectedOnly:isSelected using:FBXEXP
				)
				fetchMaxFile quiet:true
				messageBox "Animations have been exported to the scene folder based on the scene time tags." title:""
			)
			else
			(
				messageBox "Error: No time tags found in the scene." title:""
			)
		)
		
		on btn_exportTimeTags pressed do
		(
			try
			(
				local a_objs = selection as array
				if a_objs.count > 0 then --// Export only what is selected
				(
					f_exportTimeTags isSelected:true
				)
				else --// Export the whole scene
				(
					f_exportTimeTags isSelected:false
				)
			)catch(messagebox "Error: Time tags may be invalid." title:"")
		)
	) --// end rollout
		
	rollout timeTags_Rollout "Time Tags"
	(
		local startVal
		local endVal
		
		button btn_createTextFile "Create Text File" align:#left offset:[-5,0] across:3 tooltip:"Create a new text file in a chosen folder"
		button btn_openTextFile "Open Text File" align:#right offset:[48,0] tooltip:"Open a text file containing ranges in the correct format"
		button btn_ttInfo "?" width:18 height:18 offset:[30,2] tooltip:"Displays a description of how scene time tags and text file ranges should be formatted for loading"
		group ""
		(
			button btn_rangesToScene "Ranges > Scene" width:98 align:#left offset:[-3,0] across:2 tooltip:"Put the ranges of the loaded set into the current scene as time tags"
			button btn_sceneToRanges "Scene > Ranges" width:98 align:#right offset:[3,0] tooltip:"Put the time tags in the current scene as ranges of the loaded set"
			button btn_rangesToFile "Ranges > Text File" width:98 align:#left offset:[-3,0] across:2 tooltip:"Put the ranges of the loaded set to a text file"
			button btn_fileToRanges "Text File > Ranges" width:98 align:#right offset:[3,0] across:2 tooltip:"Put the time tags from a text file as ranges of the loaded set"
			button btn_sceneToFile "Scene > Text File" width:98 align:#left offset:[-3,0] across:2 tooltip:"Put the current scene time tags to a text file"
			button btn_fileToScene "Text File > Scene" width:98 align:#right offset:[3,0] tooltip:"Put the time tags from a text file into the current scene"
		)
		button btn_deleteTags ">> Delete Scene Time Tags <<" width:200 height:22 tooltip:"Remove any existing scene time tags"
		group "Add New Tags"
		(
			edittext edtxt_name "Tag Name: " fieldwidth:135
			edittext edtxt_start "" readonly:true across:2 width:65 align:#center offset:[-2,0]
			edittext edtxt_end "" readonly:true width:65 offset:[14,0]
			button btn_start "Start" across:2 width:60 height:25 align:#center tooltip:"Set the START time for a new sequence"
			button btn_end "End" enabled:false width:60 height:25 tooltip:"Set the END time for a new sequence"
		)
	
		fn f_replaceSpacesWithUnderscores str =
		(
			local str_new = ""
			local a_str = filterString str " "
			for i = 1 to a_str.count do
			(
				if i != a_str.count then
				(
					append str_new (a_str[i] + @"_")
				)
				else
				(
					append str_new a_str[i]
				)
			)
			return str_new
		)
		
		on btn_deleteTags pressed do
		(
			if (queryBox "Continue to delete scene time tags?" title:"LBT Prompt") do
			(
				FrameTagManager.ResetFrametags()
				messageBox "Time tags have been deleted from the scene." title:"LBT Prompt"
			)
		)
		
		on btn_start pressed do
		(
			startVal = sliderTime
			btn_end.enabled = true
			btn_start.enabled = false
			edtxt_start.text = (startVal as integer / ticksperframe) as string
		)
		
		on btn_end pressed do
		(
			if startVal < sliderTime then
			(
				endVal = sliderTime
				local txtName = ""
				if edtxt_name.text != "" then
				(
					txtName = edtxt_name.text
					txtName = f_replaceSpacesWithUnderscores txtName
				)
				else
				(
					txtName = "[seq] "
				)
				FrameTagManager.CreateNewTag (txtName + " " + ((startVal as integer / ticksperframe) as string) + "-" + ((endVal as integer / ticksperframe) as string)) startVal
				btn_end.enabled = false
				btn_start.enabled = true
				edtxt_end.text = (endVal as integer / ticksperframe) as string
			)
			else
			(
				messageBox "Error: The end time must be greater than the start time and current slider time." title:"CAT Error"
			)
		)
		
		on btn_rangesToScene pressed do
		(
			if queryBox "WARNING: This will replace all current time tags with the ranges from the loaded set... continue?" title:"" do
			(
				FrameTagManager.ResetFrameTags()
				for i = 1 to g_LBTools_TRM.a_ranges.count do
				(
					local a_rangeGot = g_LBTools_TRM.a_ranges[i]
					--// Make sure the name string has no spaces in it, replace them with an underscore
					local a_name = filterString a_rangeGot[1] " "
					local nameStr = ""; for j = 1 to a_name.count do (append nameStr a_name[j]; if j < a_name.count do append nameStr "_")
					--// Combine the name string with time strings then add as a time tag
					local rangeStr = nameStr + " " + a_rangeGot[2] + "-" + a_rangeGot[3]
					FrameTagManager.CreateNewTag rangeStr (a_rangeGot[2] as time)
				)
				messageBox "Process complete... all time tags in scene replaced." title:""
			)
		)
		
		on btn_sceneToRanges pressed do
		(
			local nTags = FrameTagManager.GetTagCount()
			if nTags > 0 then
			(
				if g_LBTools_TRM.f_validateSceneTags nTags then
				(
					if queryBox "WARNING: This will replace all ranges in the loaded set with the current time tags... continue?" title:"" do
					(
						delIniSetting g_LBTools_TRM.projectIniFile g_LBTools_TRM.setTag
						setIniSetting g_LBTools_TRM.projectIniFile g_LBTools_TRM.setTag "SetName" g_LBTools_TRM.setName
						setIniSetting g_LBTools_TRM.projectIniFile g_LBTools_TRM.setTag "CurrentRange" "1"
						setIniSetting g_LBTools_TRM.projectIniFile g_LBTools_TRM.setTag "RangeCount" (nTags as string)
						for i = 1 to nTags do
						(
							--// Get each time tag in turn
							local id = FrameTagManager.GetTagID i
							local tagName = FrameTagManager.GetNameByID id
							local a_tagStr = #()
							local a_rangeStr = #()
							a_tagStr = filterString tagName " "
							a_rangeStr = filterString a_tagStr[2] "-"
							local tagStr = ""
							tagStr = a_tagStr[1] + ":" + a_rangeStr[1] + ":" + a_rangeStr[2]
							--// Write to INI file
							setIniSetting g_LBTools_TRM.projectIniFile g_LBTools_TRM.setTag ("Range_" + i as string) tagStr
						)
						g_LBTools_TRM.RangeCollections_Rollout.ddl_Ranges.items = g_LBTools_TRM.f_getRangesFromFile g_LBTools_TRM.setTag
						g_LBTools_TRM.currentRange = g_LBTools_TRM.RangeCollections_Rollout.ddl_Ranges.items.count
						g_LBTools_TRM.f_sortSets()
						local num = getIniSetting g_LBTools_TRM.projectIniFile "Sets" "SetCount"
						setIniSetting g_LBTools_TRM.projectIniFile "Sets" "CurrentSet" ("Set_" + num as string)
						--// Refresh the S button depending on whether Sets Manager was open or not
						local SMwasOpen; if g_LBTools_TRM.SetsManager_Rollout.open then SMwasOpen = true else SMwasOpen = false
						g_LBTools_TRM.f_openTimeRangeManager()
						if SMwasOpen do (g_LBTools_TRM.f_openSetsManager(); g_LBTools_TRM.RangeCollections_Rollout.chkbtn_showSets.checked = true)
					)
				) else messageBox "Error: Invalid format for time tags! Process aborted.\n\n\tClick the ? button for help." title:""
			) else messageBox "Error: No time tags were found!" title:""
		)
		
		on btn_rangesToFile pressed do
		(
			g_LBTools_TRM.savedFile = getSaveFileName caption:"Create a new text file:" types:"Text file (*.txt)"
			if g_LBTools_TRM.savedFile != undefined do
			(
				try
				(
					local ttFile = openFile g_LBTools_TRM.savedFile mode:"w"
					for i = 1 to g_LBTools_TRM.a_ranges.count do
					(
						--// Combine the name string with time strings then add to text file
						local rangeStr = g_LBTools_TRM.a_ranges[i][1] + " " + g_LBTools_TRM.a_ranges[i][2] + "-" + g_LBTools_TRM.a_ranges[i][3]
						format "%\n" rangeStr to:ttFile
					)
					close ttFile
					ShellLaunch g_LBTools_TRM.savedFile ""
				) catch (messageBox "Error: Could not open file for writing." title:"")
			)
		)
		
		on btn_fileToRanges pressed do
		(
			g_LBTools_TRM.loadedFile = getOpenFileName caption:"Open an existing text file:" types:"Text file (*.txt)"
			if g_LBTools_TRM.loadedFile != undefined do
			(
				if g_LBTools_TRM.f_validateFileTags() then
				(
					if (queryBox "This process will load all ranges from the text file, replacing any existing ones in the loaded range set... continue?" title:"") do
					(
						--// Delete all ranges in the set
						a_rangeTags = #(); a_rangeTags = getIniSetting g_LBTools_TRM.projectIniFile g_LBTools_TRM.setTag
						for i = 3 to a_rangeTags.count do delIniSetting g_LBTools_TRM.projectIniFile g_LBTools_TRM.setTag a_rangeTags[i]
						--// Create an array of all ranges with their contents, read from the text file
						local a_rangeTTags = #()					
						local ttFile = openFile g_LBTools_TRM.loadedFile
						while NOT eof ttFile do
						(
							local a_lineStr = #()
							local a_rangeStr = #()
							lineStr = readLine ttFile
							a_lineStr = filterString lineStr " "
							a_rangeStr = filterString a_lineStr[2] "-"
							local rangeStr = a_lineStr[1] + ":" + a_rangeStr[1] + ":" + a_rangeStr[2]
							append a_rangeTTags rangeStr
						)
						close ttFile
						--// Write the array to the INI file to recreate the set
						setIniSetting g_LBTools_TRM.projectIniFile g_LBTools_TRM.setTag "RangeCount" (a_rangeTTags.count as string)
						setIniSetting g_LBTools_TRM.projectIniFile g_LBTools_TRM.setTag "CurrentRange" "1"
						for i = 1 to a_rangeTTags.count do setIniSetting g_LBTools_TRM.projectIniFile g_LBTools_TRM.setTag ("Range_" + i as string) a_rangeTTags[i]
						--// Refresh the S button depending on whether Sets Manager was open or not
						local SMwasOpen; if g_LBTools_TRM.SetsManager_Rollout.open then SMwasOpen = true else SMwasOpen = false
						g_LBTools_TRM.f_openTimeRangeManager()
						if SMwasOpen do (g_LBTools_TRM.f_openSetsManager(); g_LBTools_TRM.RangeCollections_Rollout.chkbtn_showSets.checked = true)	
						messageBox "Process complete... all ranges in loaded set replaced." title:""
					)
				) else messageBox "Error: Invalid format for ranges in text file! Process aborted.\n\n\tClick the ? button for help." title:""
			)
		)
		
		on btn_sceneToFile pressed do
		(
			local nTags = FrameTagManager.GetTagCount()
			if nTags > 0 then
			(
				if g_LBTools_TRM.f_validateSceneTags nTags then
				(
					g_LBTools_TRM.savedFile = getSaveFileName caption:"Create a new text file:" types:"Text file (*.txt)"
					if g_LBTools_TRM.savedFile != undefined do g_LBTools_TRM.f_saveSceneTimeTags nTags
				) else messageBox "Error: Invalid format for time tags! Process aborted.\n\n\tClick the ? button for help." title:""
			) else messageBox "Error: No time tags were found!" title:""
		)
		
		on btn_fileToScene pressed do
		(
			g_LBTools_TRM.loadedFile = getOpenFileName caption:"Open an existing text file:" types:"Text file (*.txt)"
			if g_LBTools_TRM.loadedFile != undefined do
			(
				if g_LBTools_TRM.f_validateFileTags() then
					g_LBTools_TRM.f_loadSceneTimeTags()
				else messageBox "Error: Invalid format for ranges in text file! Process aborted.\n\n\tClick the ? button for help." title:""
			)
		)
		
		on btn_openTextFile pressed do
		(
			g_LBTools_TRM.loadedFile = getOpenFileName caption:"Open an existing text file:" types:"Text file (*.txt)"
			if g_LBTools_TRM.loadedFile != undefined do ShellLaunch g_LBTools_TRM.loadedFile ""
		)
		
		on btn_createTextFile pressed do
		(
			g_LBTools_TRM.savedFile = getSaveFileName caption:"Create a new text file:" types:"Text file (*.txt)"
			if g_LBTools_TRM.savedFile != undefined do
			(
				if (findString g_LBTools_TRM.savedFile ".txt") == undefined do
				(
					g_LBTools_TRM.savedFile = g_LBTools_TRM.savedFile + ".txt"
				)
				try
				(
					local ttFile = openFile g_LBTools_TRM.savedFile mode:"w"
					ShellLaunch g_LBTools_TRM.savedFile ""
				) catch (messageBox "Error: Could not open file for writing." title:"")
			)
		)
		
		on btn_ttInfo pressed do
		(
			local msgStr = "For ranges to load correctly - Scene time tags and ranges in a text file need to be formatted like so...\n\n" +
								"\tName [space] StartTime [dash] EndTime [eol]\n\n- A space should separate the range name and start time.\n" +
								"- You can use underscores '_' or dashes '-' in the range name.\n- The start and end times must be integers, separated by a single dash.\n" +
								"- The end time must be greater than the start time.\n\nFor example:\n\thuman_run 0-100\n\thuman-walk 50-90\n\thumanSwim 80-120\n...are all valid"
			messageBox msgStr title:""
		)
		
	) --// end rollout

	rollout ProjectsManager_Rollout "Projects"
	(
		local a_projectNames = #()
	
		listBox lb_projNames "List of Projects:" width:220 height:17 align:#left across:2
		listBox lb_projSets "Sets in selected project:" width:220 height:17 align:#right readonly:true
		button btn_projLoad "LOAD" height:18 align:#left tooltip:"Load the selected project" across:6
		edittext edtxt_projName "Name:" width:200 offset:[-25,0]
		button btn_projAdd "Add" height:18 enabled:false offset:[85,0] tooltip:"Add a new project"
		button btn_projRename "Rename" height:18 enabled:false offset:[60,0] tooltip:"Rename the selected project"
		button btn_projDelete "Delete" height:18 offset:[85,0] tooltip:"Delete the selected project"
	
		on ProjectsManager_Rollout moved pos do
		(
			g_LBTools_TRM.timeRangeFloater.pos = (pos + [-254,0])
		)
		
		on ProjectsManager_Rollout open do
		(
			g_LBTools_TRM.projectName = getIniSetting g_LBTools_TRM.trmINIFile "Loaded Project" "Name" 
			edtxt_projName.text = ""			
			local a_iniFiles = getFiles (getDir #plugcfg + "\TRM#*.ini")
			--// User names for local profiles can have fullstops in them
			for i = 1 to a_iniFiles.count do
			(
				local a_pathSplit = filterString a_iniFiles[i] "#"
				local a_fileSplit = filterString a_pathSplit[2] "."
				append a_projectNames a_fileSplit[1]
			)
			lb_projNames.items = a_projectNames
			local itemIndex = findItem a_projectNames g_LBTools_TRM.projectName
			lb_projNames.selection = itemIndex
			if lb_projNames.items.count < 2 then btn_projDelete.enabled = false else btn_projDelete.enabled = true
			--// Display all the set names
			g_LBTools_TRM.a_sets = #()
			g_LBTools_TRM.a_setItemNames = #()
			g_LBTools_TRM.a_sets = g_LBTools_TRM.f_getSetsFromFile g_LBTools_TRM.projectIniFile
			for i = 1 to g_LBTools_TRM.a_sets.count do append g_LBTools_TRM.a_setItemNames g_LBTools_TRM.a_sets[i][1]
			lb_projSets.items = g_LBTools_TRM.a_setItemNames
		)

		on ProjectsManager_Rollout close do
		(	
			g_LBTools_TRM.RangeCollections_Rollout.chkbtn_showProjects.state = false
		)

		on lb_projNames selected item do
		(
			local selectedIniFile = getDir #plugcfg + ("\TRM#" + a_projectNames[item] + ".ini")
			local a_iniSets = g_LBTools_TRM.f_getSetsFromFile selectedIniFile
			local a_selectedSetNames = #()
			for i = 1 to a_iniSets.count do append a_selectedSetNames a_iniSets[i][1]
			lb_projSets.items = a_selectedSetNames
		)
		
		on btn_projLoad pressed do
		(
			g_LBTools_TRM.projectIniFile = getDir #plugcfg + ("\TRM#" + a_projectNames[lb_projNames.selection] + ".ini")
			g_LBTools_TRM.projectName = a_projectNames[lb_projNames.selection]
			setIniSetting g_LBTools_TRM.trmINIFile "Loaded Project" "Name" g_LBTools_TRM.projectName
			g_LBTools_TRM.f_openTimeRangeManager()
			g_LBTools_TRM.f_openProjectsManager()
			g_LBTools_TRM.RangeCollections_Rollout.chkbtn_showProjects.checked = true
		)
		
		on edtxt_projName changed txt do
		(
			if edtxt_projName.text != "" then
				(btn_projAdd.enabled = true; btn_projRename.enabled = true)
			else (btn_projAdd.enabled = false; btn_projRename.enabled = false)
		)
		
		on btn_projAdd pressed do
		(
			local newProjName = edtxt_projName.text
			--// First check to make sure the project name entered is unique
			local matchFound = false
			for i = 1 to a_projectNames.count do
				if matchPattern a_projectNames[i] pattern:newProjName do (matchFound = true; exit)
			if matchFound == false then --// Name is unique, so ok to create file
			(
				local newIniFile = getDir #plugcfg + ("\TRM#" + newProjName + ".ini")
				g_LBTools_TRM.f_initializeProjects_IniFile newIniFile
				g_LBTools_TRM.f_openProjectsManager()
				g_LBTools_TRM.RangeCollections_Rollout.chkbtn_showProjects.checked = true
			)
			else
			(
				messageBox "Error: Project names must be unique." title:""
				edtxt_projName.text = ""
				btn_projAdd.enabled = false
				btn_projRename.enabled = false
			)
		)
		
		on btn_projRename pressed do
		(
			local newProjName = edtxt_projName.text
			--// First check to make sure the project name entered is unique
			local matchFound = false
			for i = 1 to a_projectNames.count do
				if matchPattern a_projectNames[i] pattern:newProjName do (matchFound = true; exit)
			if matchFound == false then --// Name is unique, so ok to create file
			(
				local oldIniFile = getDir #plugcfg + ("\TRM#" + a_projectNames[lb_projNames.selection] + ".ini")
				local newIniFile = getDir #plugcfg + ("\TRM#" + newProjName + ".ini")
				renameFile oldIniFile newIniFile
				if a_projectNames[lb_projNames.selection] == g_LBTools_TRM.projectName do --// This is the loaded project, so update the ini file with the new name
				(	
					setIniSetting g_LBTools_TRM.trmINIFile "Loaded Project" "Name" newProjName
					g_LBTools_TRM.projectIniFile = getDir #plugcfg + ("\TRM#" + newProjName + ".ini")
				)
				g_LBTools_TRM.f_openTimeRangeManager()
				g_LBTools_TRM.f_openProjectsManager()
				g_LBTools_TRM.RangeCollections_Rollout.chkbtn_showProjects.checked = true
			)
			else
			(
				messageBox "Error: Project names must be unique." title:""
				edtxt_projName.text = ""
				btn_projAdd.enabled = false
				btn_projRename.enabled = false
			)
		)
		
		on btn_projDelete pressed do
		(
			if (queryBox "Are you sure you want to delete the Project?" title:"") do
			(
				deleteFile (getDir #plugcfg + ("\TRM#" + a_projectNames[lb_projNames.selection] + ".ini"))
				if a_projectNames[lb_projNames.selection] == g_LBTools_TRM.projectName AND lb_projNames.selection == 1 then --// This was the loaded project, so set the INI name to the first one
				(
					setIniSetting g_LBTools_TRM.trmINIFile "Loaded Project" "Name" a_projectNames[2]
					g_LBTools_TRM.projectIniFile = getDir #plugcfg + ("\TRM#" + a_projectNames[2] + ".ini")
				)
				else
				(
					if a_projectNames[lb_projNames.selection] == g_LBTools_TRM.projectName AND lb_projNames.selection != 1 then
					(
						setIniSetting g_LBTools_TRM.trmINIFile "Loaded Project" "Name" a_projectNames[1]
						g_LBTools_TRM.projectIniFile = getDir #plugcfg + ("\TRM#" + a_projectNames[1] + ".ini")
					)
				)
				g_LBTools_TRM.f_openTimeRangeManager()
				g_LBTools_TRM.f_openProjectsManager()
				g_LBTools_TRM.RangeCollections_Rollout.chkbtn_showProjects.checked = true
			)
		)
		
	) --// end rollout

	rollout SetsManager_Rollout "Sets"
	(
		listBox lb_setNames "List of Sets:" width:220 height:17 align:#left across:2
		listBox lb_ranges "Ranges in selected set:" width:220 height:17 align:#right readonly:true
		button btn_loadSet "LOAD" height:18 align:#left across:6 tooltip:"Load the selected set ranges"
		edittext ededtxt_setName "Name:" width:200 offset:[-25,0]
		button btm_getmaxfile "M" width:18 height:18 offset:[76,0] tooltip:"Put current maxfile name into Set Name field"
		button btn_addSet "Add" height:18 offset:[35,0] enabled:false tooltip:"Add a new set"
		button btn_renameSet "Rename" height:18 enabled:false offset:[10,0] tooltip:"Rename the selected set"
		button btn_deleteSet "Delete" height:18 offset:[10,0] tooltip:"Delete the selected set"

		on SetsManager_Rollout moved pos do
		(
			g_LBTools_TRM.timeRangeFloater.pos = (pos + [-254,0])
		)
		
		on SetsManager_Rollout open do
		(
			ededtxt_setName.text = ""
			g_LBTools_TRM.a_sets = #()
			g_LBTools_TRM.a_setItemNames = #()
			g_LBTools_TRM.a_setItemRanges = #()
			g_LBTools_TRM.a_sets = g_LBTools_TRM.f_getSetsFromFile g_LBTools_TRM.projectIniFile
			--// Display all the set names
			for i = 1 to g_LBTools_TRM.a_sets.count do append g_LBTools_TRM.a_setItemNames g_LBTools_TRM.a_sets[i][1]
			lb_setNames.items = g_LBTools_TRM.a_setItemNames
			--// Always load the current set as selection
			local itemIndex = findItem g_LBTools_TRM.a_setItemNames g_LBTools_TRM.setName --// Because of findItem, names must be unique and should be found!
			lb_setNames.selection = itemIndex --// Must not be 0!
			--// Display the ranges for the current set
			g_LBTools_TRM.a_setItemRanges = g_LBTools_TRM.a_sets[itemIndex][2]
			lb_ranges.items = g_LBTools_TRM.a_setItemRanges
			g_LBTools_TRM.selectedTag = ("Set_" + itemIndex as string)
			if lb_setNames.items.count < 2 then btn_deleteSet.enabled = false else btn_deleteSet.enabled = true
		)

		on SetsManager_Rollout close do
		(
			g_LBTools_TRM.RangeCollections_Rollout.chkbtn_showSets.checked = false
		)

		on lb_setNames selected item do
		(
			g_LBTools_TRM.a_setItemRanges = g_LBTools_TRM.a_sets[item][2]
			lb_ranges.items = g_LBTools_TRM.a_setItemRanges
			g_LBTools_TRM.selectedTag = ("Set_" + item as string)
		)
		
		on btn_loadSet pressed do
		(
			g_LBTools_TRM.setTag = g_LBTools_TRM.a_setTags[lb_setNames.selection]
			g_LBTools_TRM.setName = g_LBTools_TRM.a_setItemNames[lb_setNames.selection]
			setIniSetting g_LBTools_TRM.projectIniFile "Sets" "CurrentSet" g_LBTools_TRM.setTag
			g_LBTools_TRM.f_openTimeRangeManager()
			g_LBTools_TRM.f_openSetsManager()
			g_LBTools_TRM.RangeCollections_Rollout.chkbtn_showSets.checked = true
			g_LBTools_TRM.f_getRangesFromFile g_LBTools_TRM.setTag
		)
		
		on ededtxt_setName changed txt do
		(
			if ededtxt_setName.text != "" then
				(btn_addSet.enabled = true; btn_renameSet.enabled = true)
			else (btn_addSet.enabled = false; btn_renameSet.enabled = false)
		)
		
		on btm_getmaxfile pressed do
		(
			local nameStr = maxFileName
			local a_str = #()
			if nameStr != "" do
			(
				a_str = filterString nameStr "."
				ededtxt_setName.text = a_str[1]
				btn_addSet.enabled = true
				btn_renameSet.enabled = true
			)
		)
		
		on btn_addSet pressed do
		(
			local newSetName = ededtxt_setName.text
			--// First check to make sure the set name entered is unique
			local matchFound = false
			for i = 1 to g_LBTools_TRM.a_setItemNames.count do
				if matchPattern g_LBTools_TRM.a_setItemNames[i] pattern:newSetName do (matchFound = true; exit)
			if matchFound == false then --// Name is unique, so ok to add set
			(
				--// Make sure the name string has no spaces in it, replace them with an underscore
				local a_name = filterString newSetName " "
				local nameStr = ""; for j = 1 to a_name.count do (append nameStr a_name[j]; if j < a_name.count do append nameStr "_")
				local setsCount = g_LBTools_TRM.a_sets.count + 1
				local addedSetTag = ("Set_" + setsCount as string)
				setIniSetting g_LBTools_TRM.projectIniFile addedSetTag "SetName" nameStr
				setIniSetting g_LBTools_TRM.projectIniFile addedSetTag "CurrentRange" "1"
				setIniSetting g_LBTools_TRM.projectIniFile addedSetTag "RangeCount" "1"
				setIniSetting g_LBTools_TRM.projectIniFile addedSetTag "Range_1" "(DEFAULT_RANGE):0:100"
				setIniSetting g_LBTools_TRM.projectIniFile "Sets" "SetCount" (setsCount as string)
				g_LBTools_TRM.f_openSetsManager()
				g_LBTools_TRM.RangeCollections_Rollout.chkbtn_showSets.checked = true
			)
			else
			(
				messageBox "Error: Set names must be unique." title:""
				ededtxt_setName.text = ""
				btn_addSet.enabled = false
				btn_renameSet.enabled = false
			)
		)
		
		on btn_renameSet pressed do
		(
			local newSetName = ededtxt_setName.text
			--// First check to make sure the set name entered is unique
			local matchFound = false
			for i = 1 to g_LBTools_TRM.a_setItemNames.count do
				if matchPattern g_LBTools_TRM.a_setItemNames[i] pattern:ededtxt_setName.text do (matchFound = true; exit)
			if matchFound == false then --// Name is unique, so ok to add set
			(
				--// Make sure the name string has no spaces in it, replace them with an underscore
				local a_name = filterString newSetName " "
				local nameStr = ""; for j = 1 to a_name.count do (append nameStr a_name[j]; if j < a_name.count do append nameStr "_")
	 			setIniSetting g_LBTools_TRM.projectIniFile g_LBTools_TRM.selectedTag "SetName" nameStr
				if g_LBTools_TRM.selectedTag == g_LBTools_TRM.setTag do --// This one is loaded so update the FR dialog
				(
					g_LBTools_TRM.setName = nameStr
					g_LBTools_TRM.f_openTimeRangeManager()	
				)
				g_LBTools_TRM.f_openSetsManager()
				g_LBTools_TRM.RangeCollections_Rollout.chkbtn_showSets.checked = true
			)
			else
			(
				messageBox "Error: Set names must be unique." title:""
				ededtxt_setName.text = ""
				btn_addSet.enabled = false
				btn_renameSet.enabled = false
			)
		)
		
		on btn_deleteSet pressed do
		(
			if (queryBox "Are you sure you want to delete the Set?" title:"") do
			(
				delIniSetting g_LBTools_TRM.projectIniFile g_LBTools_TRM.selectedTag
				local a_setContents = g_LBTools_TRM.f_sortSets()
				if g_LBTools_TRM.setTag != g_LBTools_TRM.selectedTag then --// For all non-loaded sets, substract 1 from the count
				(
					local a_setTag = filterString g_LBTools_TRM.setTag "_"
					local a_selectedTag = filterString g_LBTools_TRM.selectedTag "_"
					if a_setTag[2] as integer > a_selectedTag[2] as integer do g_LBTools_TRM.setTag = a_setTag[1] + "_" + ((a_setTag[2] as integer - 1) as string)
				)
				else g_LBTools_TRM.setTag = "Set_1" --// Loaded was deleted so reset count to 1
				setIniSetting g_LBTools_TRM.projectIniFile "Sets" "CurrentSet" g_LBTools_TRM.setTag
				setIniSetting g_LBTools_TRM.projectIniFile "Sets" "SetCount" (a_setContents.count as string)
				g_LBTools_TRM.setName = getIniSetting g_LBTools_TRM.projectIniFile g_LBTools_TRM.setTag "SetName"
				g_LBTools_TRM.f_openTimeRangeManager()
				g_LBTools_TRM.f_openSetsManager()
				g_LBTools_TRM.RangeCollections_Rollout.chkbtn_showSets.checked = true
			)
		)
		
	) --// end rollout

	rollout CompactRanges_Rollout "TRM - Loaded Ranges"
	(
		button btn_expand "" width:282 height:8 align:#left tooltip:"Closes the dialog and loads the full Time Range Manager floater"
		dropdownlist ddl_Ranges width:282 across:2 items:#()
		button btn_reloadRange "R" width:18 height:18 offset:[74,1] tooltip:"Sets the scene time range to the currently selected range"

		on CompactRanges_Rollout open do
		(
			g_LBTools_TRM.setTag = getIniSetting g_LBTools_TRM.projectIniFile "Sets" "CurrentSet"
			g_LBTools_TRM.setName = getIniSetting g_LBTools_TRM.projectIniFile g_LBTools_TRM.setTag "SetName"
			g_LBTools_TRM.currentRange = getIniSetting g_LBTools_TRM.projectIniFile g_LBTools_TRM.setTag "CurrentRange"
			local rangeStr = getIniSetting g_LBTools_TRM.projectIniFile g_LBTools_TRM.setTag ("Range_" + g_LBTools_TRM.currentRange as string)
			local a_rangeStr = filterString rangeStr ":"
			ddl_Ranges.items = g_LBTools_TRM.f_getRangesFromFile g_LBTools_TRM.setTag
			ddl_Ranges.height = 600
			ddl_Ranges.selection = g_LBTools_TRM.currentRange as integer
			setIniSetting g_LBTools_TRM.trmINIFile "Compact" "State" "true"
		)

		on CompactRanges_Rollout moved pos do
		(
			g_LBTools_TRM.TRM_pos = pos
			g_LBTools_TRM.SM_pos = (g_LBTools_TRM.TRM_pos + [254,0])
			g_LBTools_TRM.PM_pos = (g_LBTools_TRM.TRM_pos + [254,0])
			setIniSetting g_LBTools_TRM.trmINIFile "Floater" "Position" (g_LBTools_TRM.TRM_pos as string)
		)

		on btn_expand pressed do
		(
			setIniSetting g_LBTools_TRM.trmINIFile "Compact" "State" "false"
			try(destroyDialog g_LBTools_TRM.CompactRanges_Rollout)catch()
			g_LBTools_TRM.f_openTimeRangeManager()
		)

		on ddl_Ranges selected item do
		(
			local rangeStr = getIniSetting g_LBTools_TRM.projectIniFile g_LBTools_TRM.setTag ("Range_" + item as string)
			local a_rangeStr = filterString rangeStr ":"		
			--// Find a camera in the scene matching the prefix (before the first underscore) and swtich viewport to that camera if it exists
			g_LBTools_TRM.f_switchCamera a_rangeStr[1]
			animationRange = interval (a_rangeStr[2] as time) (a_rangeStr[3] as time)
			g_LBTools_TRM.currentRange = item
			setIniSetting g_LBTools_TRM.projectIniFile g_LBTools_TRM.setTag "CurrentRange" (item as string)
			completeRedraw()
		)

		on btn_reloadRange pressed do
		(
			local item = ddl_Ranges.selection
			local rangeStr = getIniSetting g_LBTools_TRM.projectIniFile g_LBTools_TRM.setTag ("Range_" + item as string)
			local a_rangeStr = filterString rangeStr ":"
			animationRange = interval (a_rangeStr[2] as time) (a_rangeStr[3] as time)
			g_LBTools_TRM.currentRange = item
			setIniSetting g_LBTools_TRM.projectIniFile g_LBTools_TRM.setTag "CurrentRange" (item as string)
			completeRedraw()	
		)
	) --// end rollout

	rollout RangeCollections_Rollout "Range Collections"
	(
		edittext edtxt_projectName "Project:" width:182 across:2 readonly:true
		checkbutton chkbtn_showProjects "P" width:18 height:18 offset:[50,0] tooltip:"Displays the Projects Dialog"
		edittext edtxt_setName "Set:" width:182 across:2 readonly:true
		checkbutton chkbtn_showSets "S" offset:[50,0] width:18 height:18 tooltip:"Displays the Sets Dialog"
		button btn_compact "" width:200 height:8 offset:[0,0] tooltip:"Closes the floater and loads a smaller dialog showing only the loaded ranges"
		group "Select Range"
		(
			bitmap borderImage fileName:g_LBTools_TRM.imageFile1 width:200 height:38 align:#center offset:[0,-2]
			dropdownlist ddl_Ranges width:185 align:#center items:#() offset:[0,-38]
			label lbl_blank "" offset:[0,-10]
		)
		
		on RangeCollections_Rollout moved pos do
		(
			g_LBTools_TRM.TRM_pos = pos
			g_LBTools_TRM.SM_pos = (g_LBTools_TRM.TRM_pos + [254,0])
			g_LBTools_TRM.PM_pos = (g_LBTools_TRM.TRM_pos + [254,0])
			SetDialogPos g_LBTools_TRM.ProjectsManager_Rollout (pos + [254,0])
			SetDialogPos g_LBTools_TRM.SetsManager_Rollout (pos + [254,0])
			setIniSetting g_LBTools_TRM.trmINIFile "Floater" "Position" (g_LBTools_TRM.TRM_pos as string)
		)
		
		on RangeCollections_Rollout open do
		(
			try(destroyDialog g_LBTools_TRM.CompactRanges_Rollout)catch()
			g_LBTools_TRM.setTag = getIniSetting g_LBTools_TRM.projectIniFile "Sets" "CurrentSet"
			g_LBTools_TRM.setName = getIniSetting g_LBTools_TRM.projectIniFile g_LBTools_TRM.setTag "SetName"
			g_LBTools_TRM.currentRange = getIniSetting g_LBTools_TRM.projectIniFile g_LBTools_TRM.setTag "CurrentRange"
			edtxt_setName.text = g_LBTools_TRM.setName
			g_LBTools_TRM.projectName = getIniSetting g_LBTools_TRM.trmINIFile "Loaded Project" "Name"
			edtxt_projectName.text = g_LBTools_TRM.projectName
			ddl_Ranges.items = g_LBTools_TRM.f_getRangesFromFile g_LBTools_TRM.setTag
			ddl_Ranges.height = 600
			ddl_Ranges.selection = g_LBTools_TRM.currentRange as integer
		)
		
		on RangeCollections_Rollout close do
		(
			try(destroyDialog g_LBTools_TRM.ProjectsManager_Rollout)catch()
			try(destroyDialog g_LBTools_TRM.SetsManager_Rollout)catch()
			if g_LBTools_TRM.FBXsettings_rollout != undefined do
			(
				if g_LBTools_TRM.FBXsettings_rollout.open do
				(
					try(destroyDialog g_LBTools_TRM.FBXsettings_rollout)catch()
				)
			)
		)
		
		on chkbtn_showProjects changed state do
		(
			if state then
			(
				try(destroyDialog g_LBTools_TRM.SetsManager_Rollout)catch()
				g_LBTools_TRM.f_openProjectsManager()
			)
			else try(destroyDialog g_LBTools_TRM.ProjectsManager_Rollout)catch()
		)

		on chkbtn_showSets changed state do
		(
			if state then
			(
				try(destroyDialog g_LBTools_TRM.ProjectsManager_Rollout)catch()
				g_LBTools_TRM.f_openSetsManager()
			)
			else try(destroyDialog g_LBTools_TRM.SetsManager_Rollout)catch()
		)
			
		on btn_compact pressed do
		(
			g_LBTools_TRM.f_openCompactDialog()
		)
		
		on ddl_Ranges selected item do
		(
			local rangeStr = getIniSetting g_LBTools_TRM.projectIniFile g_LBTools_TRM.setTag ("Range_" + item as string)
			local a_rangeStr = filterString rangeStr ":"
			g_LBTools_TRM.EditRanges_Rollout.txt_newName.text = a_rangeStr[1]
			--// Find a camera in the scene matching the prefix (before the first underscore) and swtich viewport to that camera if it exists
			g_LBTools_TRM.f_switchCamera a_rangeStr[1]
			animationRange = interval (a_rangeStr[2] as time) (a_rangeStr[3] as time)
			g_LBTools_TRM.EditRanges_Rollout.spn_fromFrame.value = a_rangeStr[2] as integer
			g_LBTools_TRM.EditRanges_Rollout.spn_toFrame.value = a_rangeStr[3] as integer
			g_LBTools_TRM.currentRange = item
			setIniSetting g_LBTools_TRM.projectIniFile g_LBTools_TRM.setTag "CurrentRange" (item as string)
			sliderTime = a_rangeStr[2] as integer
			completeRedraw()
		)
	) --// end rollout

	rollout EditRanges_Rollout "Edit Ranges"
	(
		button btn_reloadRange "Reload Range" width:90 align:#left across:3 tooltip:"Sets the scene time range to the currently selected range"
		button btn_deleteRange "Delete Range" width:85 align:#left offset:[28,0] tooltip:"Deletes the currently selected range from the set. There must be at least one range available"
		button btn_deleteIniFiles "!!!" width:18 height:18 offset:[28,1] tooltip:"Deletes all INI files containing stored data for Time Range Manager"
		button btn_addNewRange "Add New Range" width:90 align:#left across:3 tooltip:"Adds a new range to the current set, based on Name + Start and End spinners"
		button btn_updateRange "Update Range" width:85 offset:[38,0] tooltip:"The selected range will update to become the entered name and spinner settings"
		button btn_addFullRange "F" width:18 height:18 offset:[28,1] tooltip:"Adds a new range called (FULL_RANGE) based on the lowest start time and highest end time"
		edittext txt_newName "Name:" align:#left text:g_LBTools_TRM.rangeName
		spinner spn_fromFrame "Start:" range:[-100000,100000,animationrange.start] type:#integer fieldwidth:50 offset:[20,0] across:3
		checkbutton chk_setSceneRange "T" align:#center width:18 height:18 tooltip:"When toggled ON - the time range will update when the spinners either side are changed"
		spinner spn_toFrame "End:" range:[-100000,100000,animationrange.end] type:#integer fieldwidth:50
		button btn_getFromScene "Get Scene Range" width:95 align:#left across:2 tooltip:"Spinners become the current scene range"
		button btn_setSceneRange "Set Scene Range" width:95 align:#right tooltip:"Scene range becomes the spinner settings"
		
		on EditRanges_Rollout open do
		(
			local rangeStr = getIniSetting g_LBTools_TRM.projectIniFile g_LBTools_TRM.setTag ("Range_" + g_LBTools_TRM.currentRange as string)
			local a_rangeStr = filterString rangeStr ":"
			txt_newName.text = a_rangeStr[1] as string
			spn_fromFrame.value = a_rangeStr[2] as integer
			spn_toFrame.value = a_rangeStr[3] as integer
			if g_LBTools_TRM.RangeCollections_Rollout.ddl_Ranges.items.count < 2 do
			(
				btn_deleteRange.enabled = false
			)
		)

		on EditRanges_Rollout rolledUp state do
		(
			if state then
			(
				setIniSetting g_LBTools_TRM.trmINIFile "Edit Ranges" "State" "false" --// Rolled down
				setIniSetting g_LBTools_TRM.trmINIFile "Floater" "Height" "407"
			)
			else
			(
				setIniSetting g_LBTools_TRM.trmINIFile "Edit Ranges" "State" "true" --// Rolled up
				setIniSetting g_LBTools_TRM.trmINIFile "Floater" "Height" "277"
			)
		)
		
		on btn_deleteIniFiles pressed do
		(
			if (queryBox "WARNING: Are you sure you want to delete all INI files? ...all stored data will be lost!" title:"") do
			(
				local configFolder = getDir #plugcfg
				local a_iniFiles = getFiles (configFolder + @"\TRM#*.ini")
				for f in a_iniFiles do
				(
					deleteFile f
				)
				deleteFile (configFolder + "\TimeRangeManager.ini")
				--// Reinitialize the INI files with default settings
				setIniSetting g_LBTools_TRM.trmINIFile "Loaded Project" "Name" "(DEFAULT_PROJECT)"
				g_LBTools_TRM.projectName = getIniSetting g_LBTools_TRM.trmINIFile "Loaded Project" "Name"
				g_LBTools_TRM.projectIniFile = configFolder + ("\TRM#" + g_LBTools_TRM.projectName + ".ini")
				g_LBTools_TRM.f_initializeProjects_IniFile g_LBTools_TRM.projectIniFile
				g_LBTools_TRM.f_initializeTRM_IniFile()
			)
		)
		
		on btn_getFromScene pressed do
		(
			spn_fromFrame.value = animationrange.start
			spn_toFrame.value = animationrange.end
		)
		
		on btn_setSceneRange pressed do animationRange = interval spn_fromFrame.value spn_toFrame.value
		
		on spn_fromFrame changed val do
		(
			local endVal = spn_toFrame.value
			if val < endVal then if chk_setSceneRange.checked do animationRange = interval val animationRange.end
			else
			(
				val += 1; spn_toFrame.value = val
				if chk_setSceneRange.checked do animationRange = interval val animationRange.end
			)
		)
		
		on spn_toFrame changed val do
		(
			local startVal = spn_fromFrame.value
			if val > startVal then if chk_setSceneRange.checked do animationRange = interval animationRange.start val
			else
			(
				val += -1; spn_fromFrame.value = val
				if chk_setSceneRange.checked do animationRange = interval animationRange.start val 
			)
		)

		on btn_reloadRange pressed do
		(
			local item = g_LBTools_TRM.RangeCollections_Rollout.ddl_Ranges.selection
			local rangeStr = getIniSetting g_LBTools_TRM.projectIniFile g_LBTools_TRM.setTag ("Range_" + item as string)
			local a_rangeStr = filterString rangeStr ":"
			txt_newName.text = a_rangeStr[1]
			animationRange = interval (a_rangeStr[2] as time) (a_rangeStr[3] as time)
			spn_fromFrame.value = a_rangeStr[2] as integer
			spn_toFrame.value = a_rangeStr[3] as integer
			g_LBTools_TRM.currentRange = item
			setIniSetting g_LBTools_TRM.projectIniFile g_LBTools_TRM.setTag "CurrentRange" (item as string)
			completeRedraw()	
		)
		
		on txt_newName changed str do g_LBTools_TRM.rangeName = str

		on btn_addNewRange pressed do
		(
			if txt_newName.text != "" then
			(
				--// First check to make sure the set name entered is unique
				local matchFound = false
				for i = 1 to g_LBTools_TRM.a_ranges.count do
					if matchPattern g_LBTools_TRM.a_ranges[i][1] pattern:txt_newName.text do (matchFound = true; exit)
				if matchFound == false then --// Name is unique, so ok to add
				(
	    			numRanges = execute (getIniSetting g_LBTools_TRM.projectIniFile g_LBTools_TRM.setTag "RangeCount")
					if numRanges == OK do numRanges = 0
					numRanges += 1
	    			setIniSetting g_LBTools_TRM.projectIniFile g_LBTools_TRM.setTag "RangeCount" (numRanges as string)				
					local a_rangeAdd = #()
					append a_rangeAdd txt_newName.text
					append a_rangeAdd (spn_fromFrame.value as string)
					append a_rangeAdd (spn_toFrame.value as string)
					--// Make sure the name string has no spaces in it, replace them with an underscore
					local a_name = filterString a_rangeAdd[1] " "
					local nameStr = ""; for j = 1 to a_name.count do (append nameStr a_name[j]; if j < a_name.count do append nameStr "_")
					--// Bulid a new string for writing to file
					rangeAddStr = (nameStr + ":" + a_rangeAdd[2] + ":" + a_rangeAdd[3])
	    			setIniSetting g_LBTools_TRM.projectIniFile g_LBTools_TRM.setTag ("Range_" + numRanges as string) rangeAddStr
					g_LBTools_TRM.RangeCollections_Rollout.ddl_Ranges.items = g_LBTools_TRM.f_getRangesFromFile g_LBTools_TRM.setTag
					g_LBTools_TRM.currentRange = g_LBTools_TRM.RangeCollections_Rollout.ddl_Ranges.items.count
					setIniSetting g_LBTools_TRM.projectIniFile g_LBTools_TRM.setTag "CurrentRange" (g_LBTools_TRM.currentRange as string)
					--// Refresh the S button depending on whether Sets Manager was open or not
					local SMwasOpen; if g_LBTools_TRM.SetsManager_Rollout.open then SMwasOpen = true else SMwasOpen = false
					g_LBTools_TRM.f_openTimeRangeManager()
					if SMwasOpen do (g_LBTools_TRM.f_openSetsManager(); g_LBTools_TRM.RangeCollections_Rollout.chkbtn_showSets.checked = true)
				)
				else
				(
					messageBox "Error: Range names must be unique." title:""
					txt_newName.text = ""
				)
			) else messageBox "Error: Enter a name in the Name field." title:""
		)
		
		on btn_updateRange pressed do
		(
			if txt_newName.text != "" then
			(
				local a_rangeAdd = #(); if txt_newName.text == g_LBTools_TRM.rangeName then append a_rangeAdd g_LBTools_TRM.rangeName
				else append a_rangeAdd txt_newName.text
				append a_rangeAdd (spn_fromFrame.value as string)
				append a_rangeAdd (spn_toFrame.value as string)
				--// Make sure the name string has no spaces in it, replace them with an underscore
				local a_name = filterString a_rangeAdd[1] " "
				local nameStr = ""; for j = 1 to a_name.count do (append nameStr a_name[j]; if j < a_name.count do append nameStr "_")
				--// Bulid a new string for writing to file
				rangeAddStr = (nameStr + ":" + a_rangeAdd[2] + ":" + a_rangeAdd[3])
				setIniSetting g_LBTools_TRM.projectIniFile g_LBTools_TRM.setTag ("Range_" + g_LBTools_TRM.RangeCollections_Rollout.ddl_Ranges.selection as string) rangeAddStr
				g_LBTools_TRM.RangeCollections_Rollout.ddl_Ranges.items = g_LBTools_TRM.f_getRangesFromFile g_LBTools_TRM.setTag
				g_LBTools_TRM.currentRange = g_LBTools_TRM.RangeCollections_Rollout.ddl_Ranges.selection
				--// Refresh the S button depending on whether Sets Manager was open or not
				local SMwasOpen; if g_LBTools_TRM.SetsManager_Rollout.open then SMwasOpen = true else SMwasOpen = false
				g_LBTools_TRM.f_openTimeRangeManager()
				if SMwasOpen do (g_LBTools_TRM.f_openSetsManager(); g_LBTools_TRM.RangeCollections_Rollout.chkbtn_showSets.checked = true)
			) else messageBox "Error: Enter a name in the Name field." title:""
		)

		on btn_addFullRange pressed do
		(
			--// First check to make sure the range FULL is not there
			local matchFound = false
			for i = 1 to g_LBTools_TRM.a_ranges.count do
				if matchPattern g_LBTools_TRM.a_ranges[i][1] pattern:"(FULL_RANGE)" do (matchFound = true; exit)
			if matchFound == false then --// Not there, so ok to add
			(
    			numRanges = execute (getIniSetting g_LBTools_TRM.projectIniFile g_LBTools_TRM.setTag "RangeCount")
				if numRanges == OK do numRanges = 0
				numRanges += 1
    			setIniSetting g_LBTools_TRM.projectIniFile g_LBTools_TRM.setTag "RangeCount" (numRanges as string)				
				local a_rangeAdd = #()
				append a_rangeAdd "(FULL_RANGE)"
				local a_fullRange = #()
				a_fullRange = g_LBTools_TRM.f_findFullRange()
				append a_rangeAdd (a_fullRange[1]) --// Add min range
				append a_rangeAdd (a_fullRange[2]) --// Add max range
				--// Make sure the name string has no spaces in it, replace them with an underscore
				local a_name = filterString a_rangeAdd[1] " "
				local nameStr = ""; for j = 1 to a_name.count do (append nameStr a_name[j]; if j < a_name.count do append nameStr "_")
				--// Build a new string for writing to file
				rangeAddStr = (nameStr + ":" + a_rangeAdd[2] + ":" + a_rangeAdd[3])
    			setIniSetting g_LBTools_TRM.projectIniFile g_LBTools_TRM.setTag ("Range_" + numRanges as string) rangeAddStr
				g_LBTools_TRM.RangeCollections_Rollout.ddl_Ranges.items = g_LBTools_TRM.f_getRangesFromFile g_LBTools_TRM.setTag
				g_LBTools_TRM.currentRange = g_LBTools_TRM.RangeCollections_Rollout.ddl_Ranges.items.count
				setIniSetting g_LBTools_TRM.projectIniFile g_LBTools_TRM.setTag "CurrentRange" (g_LBTools_TRM.currentRange as string)
				--// Refresh the S button depending on whether Sets Manager was open or not
				local SMwasOpen; if g_LBTools_TRM.SetsManager_Rollout.open then SMwasOpen = true else SMwasOpen = false
				g_LBTools_TRM.f_openTimeRangeManager()
				if SMwasOpen do (g_LBTools_TRM.f_openSetsManager(); g_LBTools_TRM.RangeCollections_Rollout.chkbtn_showSets.checked = true)
			)
			else
			(
				messageBox "Error: A range named (FULL_RANGE) already exists." title:""
			)
		)

		on btn_deleteRange pressed do
		(
			if (queryBox "Proceed with range delete?" title:"") do
			(
				delIniSetting g_LBTools_TRM.projectIniFile g_LBTools_TRM.setTag ("Range_" + g_LBTools_TRM.RangeCollections_Rollout.ddl_Ranges.selection as string)
				--// Back up the contents of remaining ranges
				local a_rangeTags = #(); a_rangeTags = getIniSetting g_LBTools_TRM.projectIniFile g_LBTools_TRM.setTag
				local a_rangeContents = #(); for i = 4 to a_rangeTags.count do append a_rangeContents (getIniSetting g_LBTools_TRM.projectIniFile g_LBTools_TRM.setTag a_rangeTags[i])
				--// Remove all ranges from set and rebuild ranges with new contents order
				for i = 4 to a_rangeTags.count do delIniSetting g_LBTools_TRM.projectIniFile g_LBTools_TRM.setTag a_rangeTags[i]
				for i = 1 to a_rangeContents.count do setIniSetting g_LBTools_TRM.projectIniFile g_LBTools_TRM.setTag ("Range_" + i as string) a_rangeContents[i]
				setIniSetting g_LBTools_TRM.projectIniFile g_LBTools_TRM.setTag "RangeCount" (a_rangeContents.count as string)
				g_LBTools_TRM.RangeCollections_Rollout.ddl_Ranges.items = g_LBTools_TRM.f_getRangesFromFile g_LBTools_TRM.setTag
				g_LBTools_TRM.currentRange = g_LBTools_TRM.RangeCollections_Rollout.ddl_Ranges.items.count
				setIniSetting g_LBTools_TRM.projectIniFile g_LBTools_TRM.setTag "CurrentRange" (g_LBTools_TRM.currentRange as string)
				--// Refresh the S button depending on whether Sets Manager was open or not
				local SMwasOpen; if g_LBTools_TRM.SetsManager_Rollout.open then SMwasOpen = true else SMwasOpen = false
				g_LBTools_TRM.f_openTimeRangeManager()
				if SMwasOpen do (g_LBTools_TRM.f_openSetsManager(); g_LBTools_TRM.RangeCollections_Rollout.chkbtn_showSets.checked = true)
			)
		)
	) --// end rollout
	
	rollout about_Rollout "About"
	(
		local imageGuide = (getDir #usericons) + "\LBTools_Guide_Image.bmp"
		
		label lbl_author "Author - Lorne Brooks"
		hyperLink myEmail "lbtools2k@gmail.com" across:2 align:#center offset:[30,10] color:(color 0 0 255) hovercolor:(color 0 0 190) visitedcolor:(color 190 0 0) align:#center address:"mailto:lbtools2k@gmail.com"
		button btn_guide "" align:#right offset:[-30,0] images:#(imageGuide, undefined, 1, 1, 1, 1, 1, false) tooltip:"Opens an online guide explaining the tool's features."
		label lbl_version "Version 2.8 - 24 Sept 2022"
		
		on btn_guide pressed do
		(
			shellLaunch "https://sites.google.com/view/lbtools-guide/home/time-range-manager" ""
		)
	) --// end rollout
	
	-----------------------------------------------------------------------------------------
	--// Functions to open dialogs
	-----------------------------------------------------------------------------------------

	fn f_openCompactDialog =
	(
		try(closeRolloutFloater g_LBTools_TRM.timeRangeFloater)catch()
		try(destroyDialog g_LBTools_TRM.ProjectsManager_Rollout)catch()
		try(destroyDialog g_LBTools_TRM.SetsManager_Rollout)catch()
		g_LBTools_TRM.TRM_pos = execute(getIniSetting g_LBTools_TRM.trmINIFile "Floater" "Position")
		createDialog g_LBTools_TRM.CompactRanges_Rollout 320 50 pos:g_LBTools_TRM.TRM_pos style:#(#style_toolwindow, #style_sysmenu)
	)

	fn f_openProjectsManager =
	(	
		try(destroyDialog g_LBTools_TRM.ProjectsManager_Rollout)catch()
		createDialog g_LBTools_TRM.ProjectsManager_Rollout 470 282 pos:g_LBTools_TRM.PM_pos
	)

	fn f_openSetsManager =
	(
		try(destroyDialog g_LBTools_TRM.SetsManager_Rollout)catch()
		createDialog g_LBTools_TRM.SetsManager_Rollout 470 282 pos:g_LBTools_TRM.SM_pos
	)
	
	fn f_openTimeRangeManager =
	(
		try(closeRolloutFloater g_LBTools_TRM.timeRangeFloater)catch()
		local height = execute(getIniSetting g_LBTools_TRM.trmINIFile "Floater" "Height")
		local editRangesState = execute(getIniSetting g_LBTools_TRM.trmINIFile "Edit Ranges" "State")
		g_LBTools_TRM.TRM_pos = execute(getIniSetting g_LBTools_TRM.trmINIFile "Floater" "Position")
		g_LBTools_TRM.timeRangeFloater = newRolloutFloater "Time Range Manager" 232 height
		g_LBTools_TRM.timeRangeFloater.pos = g_LBTools_TRM.TRM_pos
		g_LBTools_TRM.SM_pos = (g_LBTools_TRM.TRM_pos + [254,0])
		g_LBTools_TRM.PM_pos = (g_LBTools_TRM.TRM_pos + [254,0])
		addRollout g_LBTools_TRM.RangeCollections_Rollout g_LBTools_TRM.timeRangeFloater rolledUp:false
		addRollout g_LBTools_TRM.EditRanges_Rollout g_LBTools_TRM.timeRangeFloater rolledup:editRangesState
		addRollout g_LBTools_TRM.FBXexportAnim_Rollout g_LBTools_TRM.timeRangeFloater rolledUp:true
		addRollout g_LBTools_TRM.SeparateMaxfiles_Rollout g_LBTools_TRM.timeRangeFloater rolledUp:true
		addRollout g_LBTools_TRM.timeTags_Rollout g_LBTools_TRM.timeRangeFloater rolledUp:true
		--// Hack to enable the dropdownlist in Range Collections to display on top of the bitmap
		addRollout g_LBTools_TRM.about_Rollout g_LBTools_TRM.timeRangeFloater rolledUp:false
		removeRollout g_LBTools_TRM.about_Rollout g_LBTools_TRM.timeRangeFloater
		addRollout g_LBTools_TRM.about_Rollout g_LBTools_TRM.timeRangeFloater rolledUp:true
	)
	
	fn f_initializeProjects_IniFile theIniFile =
	(
		if NOT (doesFileExist theIniFile) do --// Create a new ini file with default settings
		(
			setIniSetting theIniFile "Sets" "CurrentSet" "Set_1"
			setIniSetting theIniFile "Sets" "SetCount" "1"
			setIniSetting theIniFile "Set_1" "SetName" "(DEFAULT_SET)"
			setIniSetting theIniFile "Set_1" "CurrentRange" "1"
			setIniSetting theIniFile "Set_1" "RangeCount" "1"
			setIniSetting theIniFile "Set_1" "Range_1" "(DEFAULT_RANGE):0:100"
		)	
	)

	fn f_initializeTRM_IniFile =
	(
		local compactState = false
		if (doesFileExist g_LBTools_TRM.trmINIFile) then
		(
			g_LBTools_TRM.projectName = getIniSetting g_LBTools_TRM.trmINIFile "Loaded Project" "Name"
			if NOT (hasIniSetting g_LBTools_TRM.trmINIFile "Floater" "Height") do
			(
				setIniSetting g_LBTools_TRM.trmINIFile "Floater" "Height" "277" --// Rolled up
			)
			if NOT (hasIniSetting g_LBTools_TRM.trmINIFile "Floater" "Position") do
			(
				g_LBTools_TRM.TRM_pos = mouse.screenpos + [-75,-15]
				setIniSetting g_LBTools_TRM.trmINIFile "Floater" "Position" (g_LBTools_TRM.TRM_pos as string)
			)
			if NOT (hasIniSetting g_LBTools_TRM.trmINIFile "Edit Ranges") do
			(
				setIniSetting g_LBTools_TRM.trmINIFile "Edit Ranges" "State" "true" --// Rolled up
			)
			if (hasIniSetting g_LBTools_TRM.trmINIFile "Compact") then
				compactState = execute(getIniSetting g_LBTools_TRM.trmINIFile "Compact" "State")
			else
			(
				setIniSetting g_LBTools_TRM.trmINIFile "Compact" "State" "false"
			)
		)
		else
		(
			setIniSetting g_LBTools_TRM.trmINIFile "Loaded Project" "Name" g_LBTools_TRM.projectName
			setIniSetting g_LBTools_TRM.trmINIFile "Floater" "Height" "277"
			g_LBTools_TRM.TRM_pos = mouse.screenpos + [-75,-15]
			setIniSetting g_LBTools_TRM.trmINIFile "Floater" "Position" (g_LBTools_TRM.TRM_pos as string)
			setIniSetting g_LBTools_TRM.trmINIFile "Edit Ranges" "State" "true" --// Rolled up
			setIniSetting g_LBTools_TRM.trmINIFile "Compact" "State" "false"
		)
		g_LBTools_TRM.projectIniFile = getDir #plugcfg + ("\TRM#" + g_LBTools_TRM.projectName + ".ini")
		g_LBTools_TRM.f_initializeProjects_IniFile g_LBTools_TRM.projectIniFile
		g_LBTools_TRM.f_createFBXexportINI()
		if compactState then
			g_LBTools_TRM.f_openCompactDialog()
		else
		(
			g_LBTools_TRM.f_openTimeRangeManager()
		)
	)
	
	-----------------------------------------------------------------------------------------
	--// Initial calling function
	-----------------------------------------------------------------------------------------
	f_initializeTRM_IniFile()
)

) --// end struct

g_LBTools_TRM = s_LBTools_timeRangeManager()


/*********************************************************************************************************************************************************************/
--// MACRO CALL FOR UI
/*********************************************************************************************************************************************************************/

macroScript timeRangeManager
	category:"LB Tools"
	buttonText:"{ TRM }"
	icon:#("LBTools_TRM",1)
	toolTip:"Time Range Manager 2.8"
(
	g_LBTools_TRM.f_timeRangeManager()
)

/*********************************************************************************************************************************************************************/